@@ -2,16 +2,7 @@
 #include "NumberToString.h"  //So that I can convert numbers into strings
 #include "JSONNode.h"  //To fill in the foreward declaration
 #include "JSONWorker.h"  //For fetching and parsing and such
-
-/*
-    The point of these constants is for faster assigning, if I
-    were to constantly assign to a string literal, there would be 
-    lots of copies, but since strings are copy on write, this assignment
-    is much faster
-*/
-static const json_string CONST_TRUE(JSON_TEXT("true"));
-static const json_string CONST_FALSE(JSON_TEXT("false"));
-static const json_string CONST_NULL(JSON_TEXT("null"));
+#include "JSONGlobals.h"
 
 #ifdef JSON_UNIT_TEST
     void internalJSONNode::incinternalAllocCount(void) json_nothrow { JSONNode::incinternalAllocCount(); }
@@ -26,8 +17,8 @@ internalJSONNode::internalJSONNode(const internalJSONNode & orig) json_nothrow :
     initializeFetch(orig.fetched)
     initializeComment(orig._comment)
     initializeChildren(0){
-	   
-	   
+
+
     incinternalAllocCount();
     if (isContainer()){
 	   CHILDREN = jsonChildren::newChildren();
@@ -39,7 +30,7 @@ internalJSONNode::internalJSONNode(const internalJSONNode & orig) json_nothrow :
 	   }
     }
     #ifdef JSON_MUTEX_CALLBACKS
-	   _set_mutex(orig.mylock, false);  
+	   _set_mutex(orig.mylock, false);
     #endif
 }
 
@@ -55,9 +46,9 @@ internalJSONNode::internalJSONNode(const json_string & unparsed) json_nothrow :
     initializeMutex(0)
     initializeRefCount(1)
     initializeFetch(false)
-    initializeComment(EMPTY_JSON_STRING)
+    initializeComment(json_global(EMPTY_JSON_STRING))
     initializeChildren(0){
-	   
+
     incinternalAllocCount();
     switch (unparsed[0]){
 	   case JSON_TEXT('{'):  //node
@@ -93,11 +84,11 @@ internalJSONNode::internalJSONNode(const json_string & name_t, const json_string
     initializeMutex(0)
     initializeRefCount(1)
     initializeFetch(false)
-    initializeComment(EMPTY_JSON_STRING)
+    initializeComment(json_global(EMPTY_JSON_STRING))
     initializeChildren(0){
-	   
+
     incinternalAllocCount();
-	   
+
     #ifdef JSON_STRICT
 	   JSON_ASSERT_SAFE(!value_t.empty(), JSON_TEXT("empty node"), Nullify(NOTVALID); return;);
     #else
@@ -107,17 +98,17 @@ internalJSONNode::internalJSONNode(const json_string & name_t, const json_string
 		  return;
 	   }
     #endif
-    
+
     _string = value_t;
     #ifdef JSON_LESS_MEMORY
 	   JSON_ASSERT(_string.capacity() == _string.length(), JSON_TEXT("_string object too large"));
     #endif
-	   
+
     const json_char firstchar = value_t[0];
     #if defined JSON_DEBUG || defined JSON_SAFE
 	   const json_char lastchar = value_t[value_t.length() - 1];
     #endif
-    
+
     switch (firstchar){
         case JSON_TEXT('\"'):  //a json_string literal, still escaped and with leading and trailing quotes
             JSON_ASSERT_SAFE(lastchar == JSON_TEXT('\"'), JSON_TEXT("Unterminated quote"), Nullify(); return;);
@@ -137,24 +128,24 @@ internalJSONNode::internalJSONNode(const json_string & name_t, const json_string
 		  SetFetchedFalseOrDo(FetchArray());
             break;
         LETTERCASE('t', 'T'):
-            JSON_ASSERT_SAFE(value_t == JSON_TEXT("true"), json_string(json_string(JSON_TEXT("unknown JSON literal: ")) + value_t).c_str(), Nullify(); return;);
+            JSON_ASSERT_SAFE(value_t == json_global(CONST_TRUE), json_string(json_global(ERROR_UNKNOWN_LITERAL) + value_t).c_str(), Nullify(); return;);
             _value._bool = true;
             _type = JSON_BOOL;
 		  SetFetched(true);
             break;
         LETTERCASE('f', 'F'):
-            JSON_ASSERT_SAFE(value_t == JSON_TEXT("false"), json_string(json_string(JSON_TEXT("unknown JSON literal: ")) + value_t).c_str(), Nullify(); return;);
+            JSON_ASSERT_SAFE(value_t == json_global(CONST_FALSE), json_string(json_global(ERROR_UNKNOWN_LITERAL) + value_t).c_str(), Nullify(); return;);
             _value._bool = false;
             _type = JSON_BOOL;
 		  SetFetched(true);
             break;
         LETTERCASE('n', 'N'):
-            JSON_ASSERT_SAFE(value_t == JSON_TEXT("null"), json_string(json_string(JSON_TEXT("unknown JSON literal: ")) + value_t).c_str(), Nullify(); return;);
+            JSON_ASSERT_SAFE(value_t == json_global(CONST_NULL), json_string(json_global(ERROR_UNKNOWN_LITERAL) + value_t).c_str(), Nullify(); return;);
             _type = JSON_NULL;
 		  SetFetched(true);
             break;
         default:
-            JSON_ASSERT_SAFE(NumberToString::isNumeric(value_t), json_string(json_string(JSON_TEXT("unknown JSON literal: ")) + value_t).c_str(), Nullify(); return;);
+            JSON_ASSERT_SAFE(NumberToString::isNumeric(value_t), json_string(json_global(ERROR_UNKNOWN_LITERAL) + value_t).c_str(), Nullify(); return;);
 		  _type = JSON_NUMBER;
 		  SetFetchedFalseOrDo(FetchNumber());
             break;
@@ -166,7 +157,7 @@ internalJSONNode::internalJSONNode(const json_string & name_t, const json_string
 internalJSONNode::~internalJSONNode(void) json_nothrow {
     decinternalAllocCount();
     #ifdef JSON_MUTEX_CALLBACKS
-	   _unset_mutex();  
+	   _unset_mutex();
     #endif
     DELETE_CHILDREN();
 }
@@ -202,25 +193,44 @@ internalJSONNode::~internalJSONNode(void) json_nothrow {
 
 //This one is used by as_int and as_float, so even non-readers need it
 void internalJSONNode::FetchNumber(void) const json_nothrow {
-    #if defined(JSON_OCTAL) || defined(JSON_UNICODE)
-	   const size_t len = _string.length();
-    #endif
-
     #ifdef JSON_STRICT
 	   _value._number = NumberToString::_atof(_string.c_str())
     #else
 	   #ifdef JSON_UNICODE
-		  json_auto<char> temp(len + 1);
-		  wcstombs(temp.ptr, _string.c_str(), len);
-		  temp.ptr[len] = '\0';
-		  _value._number = (json_number)atof(temp.ptr);
+		  const size_t len = _string.length();
+		  #if defined(_MSC_VER) && defined(JSON_SAFE)
+			 const size_t bytes = (len * (sizeof(json_char) / sizeof(char))) + 1;
+			 json_auto<char> temp(bytes);
+			 size_t res;
+			 errno_t err = std::wcstombs_s(&res, temp.ptr, bytes, _string.c_str(), len);
+			 if (err != 0){
+				_value._number = (json_number)0.0;
+				return;
+			 }
+		  #elif defined(JSON_SAFE)
+			 const size_t bytes = (len * (sizeof(json_char) / sizeof(char))) + 1;
+			 json_auto<char> temp(bytes);
+			 size_t res = std::wcstombs(temp.ptr, _string.c_str(), len);
+			 if (res == (size_t)-1){
+				_value._number = (json_number)0.0;
+				return;
+			 }
+		  #else
+			 json_auto<char> temp(len + 1);
+			 size_t res = std::wcstombs(temp.ptr, _string.c_str(), len);
+		  #endif
+		  temp.ptr[res] = '\0';
+		  _value._number = (json_number)std::atof(temp.ptr);
 	   #else
-		  _value._number = (json_number)atof(_string.c_str());
+		  _value._number = (json_number)std::atof(_string.c_str());
 	   #endif
     #endif
+    #if((!defined(JSON_CASTABLE) && defined(JSON_LESS_MEMORY)) && !defined(JSON_WRITE_PRIORITY))
+	   clearString(_string);
+    #endif
 }
 
-#if !defined(JSON_PREPARSE) && defined(JSON_READ_PRIORITY) 
+#if !defined(JSON_PREPARSE) && defined(JSON_READ_PRIORITY)
     void internalJSONNode::Fetch(void) const json_nothrow {
 	   if (fetched) return;
 	   switch (type()){
@@ -242,7 +252,7 @@ void internalJSONNode::FetchNumber(void) const json_nothrow {
 				Nullify();
 		  #endif
 	   }
-	   fetched = true;  
+	   fetched = true;
     }
 #endif
 
@@ -255,11 +265,15 @@ void internalJSONNode::Set(const json_string & val) json_nothrow {
 }
 
 #ifdef JSON_LIBRARY
-    void internalJSONNode::Set(long val) json_nothrow {
+    void internalJSONNode::Set(json_int_t val) json_nothrow {
 	   makeNotContainer();
 	   _type = JSON_NUMBER;
 	   _value._number = (json_number)val;
-	   _string = NumberToString::_itoa<long>(val);
+	   #if(defined(JSON_CASTABLE) || !defined(JSON_LESS_MEMORY) || defined(JSON_WRITE_PRIORITY))
+		  _string = NumberToString::_itoa<json_int_t>(val);
+	   #else 
+		  clearString(_string);
+	   #endif
 	   SetFetched(true);
     }
 
@@ -267,27 +281,50 @@ void internalJSONNode::Set(const json_string & val) json_nothrow {
 	   makeNotContainer();
 	   _type = JSON_NUMBER;
 	   _value._number = val;
-	   _string = NumberToString::_ftoa(val);
+	   #if(defined(JSON_CASTABLE) || !defined(JSON_LESS_MEMORY) || defined(JSON_WRITE_PRIORITY))
+		  _string = NumberToString::_ftoa(val);
+	   #else 
+		  clearString(_string);
+	   #endif
 	   SetFetched(true);
     }
 #else
-    #define SET(converter, type)\
-	   void internalJSONNode::Set(type val) json_nothrow {\
-		  makeNotContainer();\
-		  _type = JSON_NUMBER;\
-		  _value._number = (json_number)val;\
-		  _string = NumberToString::converter<type>(val);\
-		  SetFetched(true);\
-	   }
+    #if(defined(JSON_CASTABLE) || !defined(JSON_LESS_MEMORY) || defined(JSON_WRITE_PRIORITY))
+	   #define SET(converter, type)\
+		  void internalJSONNode::Set(type val) json_nothrow {\
+			 makeNotContainer();\
+			 _type = JSON_NUMBER;\
+			 _value._number = (json_number)val;\
+			 _string = NumberToString::converter<type>(val);\
+			 SetFetched(true);\
+		  }
+	   #define SET_FLOAT(type) \
+		  void internalJSONNode::Set(type val) json_nothrow {\
+			 makeNotContainer();\
+			 _type = JSON_NUMBER;\
+			 _value._number = (json_number)val;\
+			 _string = NumberToString::_ftoa(_value._number);\
+			 SetFetched(true);\
+		  }
+    #else
+	   #define SET(converter, type)\
+		  void internalJSONNode::Set(type val) json_nothrow {\
+			 makeNotContainer();\
+			 _type = JSON_NUMBER;\
+			 _value._number = (json_number)val;\
+			 clearString(_string);\
+			 SetFetched(true);\
+		  }
+	   #define SET_FLOAT(type) \
+		  void internalJSONNode::Set(type val) json_nothrow {\
+			 makeNotContainer();\
+			 _type = JSON_NUMBER;\
+			 _value._number = (json_number)val;\
+			 clearString(_string);\
+			 SetFetched(true);\
+		  }
+    #endif
     #define SET_INTEGER(type) SET(_itoa, type) SET(_uitoa, unsigned type)
-    #define SET_FLOAT(type) \
-	   void internalJSONNode::Set(type val) json_nothrow {\
-		  makeNotContainer();\
-		  _type = JSON_NUMBER;\
-		  _value._number = (json_number)val;\
-		  _string = NumberToString::_ftoa(_value._number);\
-		  SetFetched(true);\
-	   }
 
     SET_INTEGER(char)
     SET_INTEGER(short)
@@ -295,6 +332,7 @@ void internalJSONNode::Set(const json_string & val) json_nothrow {
     SET_INTEGER(long)
     #ifndef JSON_ISO_STRICT
 	   SET_INTEGER(long long)
+	   SET_FLOAT(long double)
     #endif
 
     SET_FLOAT(float)
@@ -305,7 +343,9 @@ void internalJSONNode::Set(bool val) json_nothrow {
     makeNotContainer();
     _type = JSON_BOOL;
     _value._bool = val;
-    _string = val ? CONST_TRUE : CONST_FALSE;
+    #if(defined(JSON_CASTABLE) || !defined(JSON_LESS_MEMORY) || defined(JSON_WRITE_PRIORITY))
+	   _string = val ? json_global(CONST_TRUE) : json_global(CONST_FALSE);
+    #endif
     SetFetched(true);
 }
 
@@ -314,7 +354,7 @@ bool internalJSONNode::IsEqualTo(const internalJSONNode * val) const json_nothro
     if (type() != val -> type()) return false;	 //aren't even same type
     if (_name != val -> _name) return false;  //names aren't the same
     if (type() == JSON_NULL) return true;  //both null, can't be different
-    #if !defined(JSON_PREPARSE) && defined(JSON_READ_PRIORITY) 
+    #if !defined(JSON_PREPARSE) && defined(JSON_READ_PRIORITY)
 	   Fetch();
 	   val -> Fetch();
     #endif
@@ -326,15 +366,15 @@ bool internalJSONNode::IsEqualTo(const internalJSONNode * val) const json_nothro
 	   case JSON_BOOL:
 		  return val -> _value._bool == _value._bool;
     };
-    
+
     JSON_ASSERT(type() == JSON_NODE || type() == JSON_ARRAY, JSON_TEXT("Checking for equality, not sure what type"));
     if (CHILDREN -> size() != val -> CHILDREN -> size()) return false;  //if they arne't he same size then they certainly aren't equal
-    
+
     //make sure each children is the same
     JSONNode ** valrunner = val -> CHILDREN -> begin();
     json_foreach(CHILDREN, myrunner){
-        JSON_ASSERT(*myrunner, JSON_TEXT("a null pointer within the children"));
-	   JSON_ASSERT(*valrunner, JSON_TEXT("a null pointer within the children"));
+        JSON_ASSERT(*myrunner != NULL, json_global(ERROR_NULL_IN_CHILDREN));
+	   JSON_ASSERT(*valrunner != NULL, json_global(ERROR_NULL_IN_CHILDREN));
 	   JSON_ASSERT(valrunner != val -> CHILDREN -> end(), JSON_TEXT("at the end of other one's children, but they're the same size?"));
         if (**myrunner != **valrunner) return false;
 	   ++valrunner;
@@ -344,10 +384,14 @@ bool internalJSONNode::IsEqualTo(const internalJSONNode * val) const json_nothro
 
 void internalJSONNode::Nullify(void) const json_nothrow {
     _type = JSON_NULL;
-    _string = CONST_NULL;
+    #if(defined(JSON_CASTABLE) || !defined(JSON_LESS_MEMORY) || defined(JSON_WRITE_PRIORITY))
+	   _string = json_global(CONST_NULL);
+    #else
+	   clearString(_string);
+    #endif
     SetFetched(true);
 }
-    
+
 #ifdef JSON_MUTEX_CALLBACKS
     #define JSON_MUTEX_COPY ,mylock
 #else
@@ -359,7 +403,7 @@ void internalJSONNode::push_back(JSONNode * node) json_nothrow {
 #else
 void internalJSONNode::push_back(const JSONNode & node) json_nothrow {
 #endif
-    JSON_ASSERT_SAFE(isContainer(), JSON_TEXT("calling push_back on non-container type"), return;);
+    JSON_ASSERT_SAFE(isContainer(), json_global(ERROR_NON_CONTAINER) + JSON_TEXT("push_back"), return;);
     #ifdef JSON_LIBRARY
 	   #ifdef JSON_MUTEX_CALLBACKS
 		  if (mylock != 0) node -> set_mutex(mylock);
@@ -369,14 +413,14 @@ void internalJSONNode::push_back(const JSONNode & node) json_nothrow {
 	   CHILDREN -> push_back(JSONNode::newJSONNode(node   JSON_MUTEX_COPY));
     #endif
 }
-    
+
 void internalJSONNode::push_front(const JSONNode & node) json_nothrow {
-    JSON_ASSERT_SAFE(isContainer(), JSON_TEXT("calling at on non-container type"), return;);
+    JSON_ASSERT_SAFE(isContainer(), json_global(ERROR_NON_CONTAINER) + JSON_TEXT("push_front"), return;);
     CHILDREN -> push_front(JSONNode::newJSONNode(node   JSON_MUTEX_COPY));
 }
 
 JSONNode * internalJSONNode::pop_back(json_index_t pos) json_nothrow {
-    JSON_ASSERT_SAFE(isContainer(), JSON_TEXT("calling at on non-container type"), return 0;);
+    JSON_ASSERT_SAFE(isContainer(), json_global(ERROR_NON_CONTAINER) + JSON_TEXT("pop_back"), return 0;);
     JSONNode * result = (*CHILDREN)[pos];
     JSONNode ** temp = CHILDREN -> begin() + pos;
     CHILDREN -> erase(temp);
@@ -384,7 +428,7 @@ JSONNode * internalJSONNode::pop_back(json_index_t pos) json_nothrow {
 }
 
 JSONNode * internalJSONNode::pop_back(const json_string & name_t) json_nothrow {
-    JSON_ASSERT_SAFE(isContainer(), JSON_TEXT("calling pop_back (named) on non-container type"), return 0;);
+    JSON_ASSERT_SAFE(isContainer(), json_global(ERROR_NON_CONTAINER) + JSON_TEXT("pop_back(str)"), return 0;);
     if (JSONNode ** res = at(name_t)){
 	   JSONNode * result = *res;
 	   CHILDREN -> erase(res);
@@ -395,7 +439,7 @@ JSONNode * internalJSONNode::pop_back(const json_string & name_t) json_nothrow {
 
 #ifdef JSON_CASE_INSENSITIVE_FUNCTIONS
     JSONNode * internalJSONNode::pop_back_nocase(const json_string & name_t) json_nothrow {
-	   JSON_ASSERT_SAFE(isContainer(), JSON_TEXT("calling pop_back_nocase on non-container type"), return 0;);
+	   JSON_ASSERT_SAFE(isContainer(), json_global(ERROR_NON_CONTAINER) + JSON_TEXT("pop_back_nocase"), return 0;);
 	   if (JSONNode ** res = at_nocase(name_t)){
 		  JSONNode * result = *res;
 		  CHILDREN -> erase(res);
@@ -406,10 +450,10 @@ JSONNode * internalJSONNode::pop_back(const json_string & name_t) json_nothrow {
 #endif
 
 JSONNode ** internalJSONNode::at(const json_string & name_t) json_nothrow {
-    JSON_ASSERT_SAFE(isContainer(), JSON_TEXT("calling at on non-container type"), return 0;);
+    JSON_ASSERT_SAFE(isContainer(), json_global(ERROR_NON_CONTAINER) + JSON_TEXT("at"), return 0;);
     Fetch();
     json_foreach(CHILDREN, myrunner){
-	   JSON_ASSERT(*myrunner, JSON_TEXT("a null pointer within the children"));
+	   JSON_ASSERT(*myrunner != NULL, json_global(ERROR_NULL_IN_CHILDREN));
 	   if (json_unlikely((*myrunner) -> name() == name_t)) return myrunner;
     }
     return 0;
@@ -431,23 +475,23 @@ JSONNode ** internalJSONNode::at(const json_string & name_t) json_nothrow {
 		  }
 		  ++ch_one;
 		  ++ch_two;
-		  
+
 	   }
 	   return *ch_two == '\0';  //this one has to be null terminated too, or else json_string two is longer, hence, not equal
     }
 
     JSONNode ** internalJSONNode::at_nocase(const json_string & name_t) json_nothrow {
-	   JSON_ASSERT_SAFE(isContainer(), JSON_TEXT("calling at_nocase on non-container type"), return 0;);
+	   JSON_ASSERT_SAFE(isContainer(), json_global(ERROR_NON_CONTAINER) + JSON_TEXT("at_nocase"), return 0;);
 	   Fetch();
 	   json_foreach(CHILDREN, myrunner){
-		  JSON_ASSERT(*myrunner, JSON_TEXT("a null pointer within the children"));
+		  JSON_ASSERT(*myrunner, json_global(ERROR_NULL_IN_CHILDREN));
 		  if (json_unlikely(AreEqualNoCase((*myrunner) -> name().c_str(), name_t.c_str()))) return myrunner;
 	   }
 	   return 0;
     }
 #endif
 
-#if !defined(JSON_PREPARSE) && defined(JSON_READ_PRIORITY) 
+#if !defined(JSON_PREPARSE) && defined(JSON_READ_PRIORITY)
     void internalJSONNode::preparse(void) json_nothrow {
 	   Fetch();
 	   if (isContainer()){
@@ -458,13 +502,237 @@ JSONNode ** internalJSONNode::at(const json_string & name_t) json_nothrow {
     }
 #endif
 
+internalJSONNode::operator bool() const json_nothrow {
+    Fetch();
+    #ifdef JSON_CASTABLE
+	   switch(type()){
+		  case JSON_NUMBER:
+			 return !_floatsAreEqual(_value._number, (json_number)0.0);
+		  case JSON_NULL:
+			 return false;
+	   }
+    #endif
+    JSON_ASSERT(type() == JSON_BOOL, json_global(ERROR_UNDEFINED) + JSON_TEXT("(bool)"));
+    return _value._bool;
+}
+
+#ifdef JSON_LIBRARY
+    internalJSONNode::operator json_number() const json_nothrow {
+	   Fetch();
+	   #ifdef JSON_CASTABLE
+		  switch(type()){
+			 case JSON_NULL:
+				return (json_number)0.0;
+			 case JSON_BOOL:
+				return (json_number)(_value._bool ? 1.0 : 0.0);
+			 case JSON_STRING:
+				FetchNumber();
+		  }
+	   #endif
+	   JSON_ASSERT(type() == JSON_NUMBER, json_global(ERROR_UNDEFINED) + JSON_TEXT("as_float"));
+	   return (json_number)_value._number;
+    }
+
+    internalJSONNode::operator json_int_t() const json_nothrow {
+	   Fetch();
+	   #ifdef JSON_CASTABLE
+		  switch(type()){
+			 case JSON_NULL:
+				return 0;
+			 case JSON_BOOL:
+				return _value._bool ? 1 : 0;
+			 case JSON_STRING:
+				FetchNumber();
+		  }
+	   #endif
+	   JSON_ASSERT(type() == JSON_NUMBER, json_global(ERROR_UNDEFINED) + JSON_TEXT("as_int"));
+	   JSON_ASSERT(_value._number == (json_number)((json_int_t)_value._number), json_string(JSON_TEXT("as_int will truncate ")) + _string);
+	   return (json_int_t)_value._number;
+    }
+#else
+    #ifndef JSON_ISO_STRICT
+	   internalJSONNode::operator long double() const json_nothrow {
+		  Fetch();
+		  #ifdef JSON_CASTABLE
+			 switch(type()){
+				case JSON_NULL:
+				    return (long double)0.0;
+				case JSON_BOOL:
+				    return (long double)(_value._bool ? 1.0 : 0.0);
+				case JSON_STRING:
+				    FetchNumber();
+			 }
+		  #endif
+		  JSON_ASSERT(type() == JSON_NUMBER, json_global(ERROR_UNDEFINED) + JSON_TEXT("(long double)"));
+		  return (long double)_value._number;
+	   }
+    #else
+	   internalJSONNode::operator double() const json_nothrow {
+		  Fetch();
+		  #ifdef JSON_CASTABLE
+			 switch(type()){
+				case JSON_NULL:
+				    return (double)0.0;
+				case JSON_BOOL:
+				    return (double)(_value._bool ? 1.0 : 0.0);
+				case JSON_STRING:
+				    FetchNumber();
+			 }
+		  #endif
+		  JSON_ASSERT(type() == JSON_NUMBER, json_global(ERROR_UNDEFINED) + JSON_TEXT("(double)"));
+		  return (double)_value._number;
+	   }
+    #endif
+
+    //do whichever one is longer, because it's easy to cast down
+    #ifdef JSON_ISO_STRICT
+	   internalJSONNode::operator long() const json_nothrow
+    #else
+	   internalJSONNode::operator long long() const json_nothrow
+    #endif
+    {
+	   Fetch();
+	   #ifdef JSON_CASTABLE
+		  switch(type()){
+			 case JSON_NULL:
+				return 0;
+			 case JSON_BOOL:
+				return _value._bool ? 1 : 0;
+			 case JSON_STRING:
+				FetchNumber();
+		  }
+	   #endif
+	   #ifdef JSON_ISO_STRICT
+		  JSON_ASSERT(type() == JSON_NUMBER, json_global(ERROR_UNDEFINED) + JSON_TEXT("(long)"));
+		  JSON_ASSERT(_value._number > LONG_MIN, _string + json_global(ERROR_LOWER_RANGE) + JSON_TEXT("long"));
+		  JSON_ASSERT(_value._number < LONG_MAX, _string + json_global(ERROR_UPPER_RANGE) + JSON_TEXT("long"));
+		  JSON_ASSERT(_value._number == (json_number)((long)_value._number), json_string(JSON_TEXT("(long) will truncate ")) + _string);
+		  return (long)_value._number;
+	   #else
+		  JSON_ASSERT(type() == JSON_NUMBER, json_global(ERROR_UNDEFINED) + JSON_TEXT("(long long)"));
+		  #ifdef LONG_LONG_MAX			 
+			 JSON_ASSERT(_value._number < LONG_LONG_MAX, _string + json_global(ERROR_UPPER_RANGE) + JSON_TEXT("long long"));
+		  #elif defined(LLONG_MAX)
+			 JSON_ASSERT(_value._number < LLONG_MAX, _string + json_global(ERROR_UPPER_RANGE) + JSON_TEXT("long long"));
+		  #endif
+		  #ifdef LONG_LONG_MIN
+			 JSON_ASSERT(_value._number > LONG_LONG_MIN, _string + json_global(ERROR_LOWER_RANGE) + JSON_TEXT("long long"));
+		  #elif defined(LLONG_MAX)
+			 JSON_ASSERT(_value._number > LLONG_MIN, _string + json_global(ERROR_LOWER_RANGE) + JSON_TEXT("long long"));
+		  #endif
+
+		  JSON_ASSERT(_value._number == (json_number)((long long)_value._number), json_string(JSON_TEXT("(long long) will truncate ")) + _string);
+		  return (long long)_value._number;
+	   #endif
+    }
+
+    #ifdef JSON_ISO_STRICT
+	   internalJSONNode::operator unsigned long() const json_nothrow
+    #else
+	   internalJSONNode::operator unsigned long long() const json_nothrow
+    #endif
+    {
+	   Fetch();
+	   #ifdef JSON_CASTABLE
+		  switch(type()){
+			 case JSON_NULL:
+				return 0;
+			 case JSON_BOOL:
+				return _value._bool ? 1 : 0;
+			 case JSON_STRING:
+				FetchNumber();
+		  }
+	   #endif
+	   #ifdef JSON_ISO_STRICT
+		  JSON_ASSERT(type() == JSON_NUMBER, json_global(ERROR_UNDEFINED) + JSON_TEXT("(unsigned long)"));
+		  JSON_ASSERT(_value._number > 0, _string + json_global(ERROR_LOWER_RANGE) + JSON_TEXT("unsigned long"));
+		  JSON_ASSERT(_value._number < ULONG_MAX, _string + json_global(ERROR_UPPER_RANGE) + JSON_TEXT("unsigned long"));
+		  JSON_ASSERT(_value._number == (json_number)((unsigned long)_value._number), json_string(JSON_TEXT("(unsigend long) will truncate ")) + _string);
+		  return (unsigned long)_value._number;
+	   #else
+		  JSON_ASSERT(type() == JSON_NUMBER, json_global(ERROR_UNDEFINED) + JSON_TEXT("(unsigned long long)"));
+		  JSON_ASSERT(_value._number > 0, _string + json_global(ERROR_LOWER_RANGE) + JSON_TEXT("unsigned long long"));
+		  #ifdef ULONG_LONG_MAX
+			 JSON_ASSERT(_value._number < ULONG_LONG_MAX, _string + json_global(ERROR_UPPER_RANGE) + JSON_TEXT("unsigned long long"));
+		  #elif defined(ULLONG_MAX)
+			 JSON_ASSERT(_value._number < ULLONG_MAX, _string + json_global(ERROR_UPPER_RANGE) + JSON_TEXT("unsigned long long"));
+		  #endif
+		  JSON_ASSERT(_value._number == (json_number)((unsigned long long)_value._number), json_string(JSON_TEXT("(unsigned long long) will truncate ")) + _string);
+		  return (unsigned long long)_value._number;
+	   #endif
+    }
+#endif
+	
+	/*
+	 These functions are to allow allocation to be completely controlled by the callbacks
+	 */
+	
+#ifdef JSON_MEMORY_POOL
+	#include "JSONMemoryPool.h"
+	static memory_pool<INTERNALNODEPOOL> json_internal_mempool;
+#endif
+	
+void internalJSONNode::deleteInternal(internalJSONNode * ptr) json_nothrow {
+	#ifdef JSON_MEMORY_POOL
+		ptr -> ~internalJSONNode();
+		json_internal_mempool.deallocate((void*)ptr);
+	#elif defined(JSON_MEMORY_CALLBACKS)
+		ptr -> ~internalJSONNode();
+		libjson_free<internalJSONNode>(ptr);
+	#else
+		delete ptr;
+	#endif
+}
+	
+internalJSONNode * internalJSONNode::newInternal(char mytype) {
+	#ifdef JSON_MEMORY_POOL
+		return new((internalJSONNode*)json_internal_mempool.allocate()) internalJSONNode(mytype);
+	#elif defined(JSON_MEMORY_CALLBACKS)
+		return new(json_malloc<internalJSONNode>(1)) internalJSONNode(mytype);
+	#else
+		return new internalJSONNode(mytype);
+	#endif
+}
+	
+#ifdef JSON_READ_PRIORITY
+internalJSONNode * internalJSONNode::newInternal(const json_string & unparsed) {
+	#ifdef JSON_MEMORY_POOL
+		return new((internalJSONNode*)json_internal_mempool.allocate()) internalJSONNode(unparsed);
+	#elif defined(JSON_MEMORY_CALLBACKS)
+		return new(json_malloc<internalJSONNode>(1)) internalJSONNode(unparsed);
+	#else
+		return new internalJSONNode(unparsed);
+	#endif
+}
+	
+internalJSONNode * internalJSONNode::newInternal(const json_string & name_t, const json_string & value_t) {
+	#ifdef JSON_MEMORY_POOL
+		return new((internalJSONNode*)json_internal_mempool.allocate()) internalJSONNode(name_t, value_t);
+	#elif defined(JSON_MEMORY_CALLBACKS)
+		return new(json_malloc<internalJSONNode>(1)) internalJSONNode(name_t, value_t);
+	#else
+		return new internalJSONNode(name_t, value_t);
+	#endif
+}
+#endif
+	
+internalJSONNode * internalJSONNode::newInternal(const internalJSONNode & orig) {
+	#ifdef JSON_MEMORY_POOL
+		return new((internalJSONNode*)json_internal_mempool.allocate()) internalJSONNode(orig);
+	#elif defined(JSON_MEMORY_CALLBACKS)
+		return new(json_malloc<internalJSONNode>(1)) internalJSONNode(orig);
+	#else
+		return new internalJSONNode(orig);
+	#endif
+}
+
 #ifdef JSON_DEBUG
     #ifndef JSON_LIBRARY
 	   JSONNode internalJSONNode::Dump(size_t & totalbytes) const json_nothrow {
 		  JSONNode dumpage(JSON_NODE);
 		  dumpage.set_name(JSON_TEXT("internalJSONNode"));
 		  dumpage.push_back(JSON_NEW(JSONNode(JSON_TEXT("this"), (long)this)));
-		  
+
 		  START_MEM_SCOPE
 			 size_t memory = sizeof(internalJSONNode);
 			 memory += _name.capacity() * sizeof(json_char);
@@ -479,8 +747,8 @@ JSONNode ** internalJSONNode::at(const json_string & name_t) json_nothrow {
 			 totalbytes += memory;
 			 dumpage.push_back(JSON_NEW(JSONNode(JSON_TEXT("bytes used"), memory)));
 		  END_MEM_SCOPE
-		  
-		  
+
+
 		  #ifdef JSON_REF_COUNT
 			 dumpage.push_back(JSON_NEW(JSONNode(JSON_TEXT("refcount"), refcount)));
 		  #endif
@@ -493,7 +761,7 @@ JSONNode ** internalJSONNode::at(const json_string & name_t) json_nothrow {
 			 case ty:\
 				dumpage.push_back(JSON_NEW(JSONNode(JSON_TEXT("_type"), JSON_TEXT(#ty))));\
 				break;
-		  
+
 		  switch(type()){
 			 DUMPCASE(JSON_NULL)
 			 DUMPCASE(JSON_STRING)
@@ -504,13 +772,13 @@ JSONNode ** internalJSONNode::at(const json_string & name_t) json_nothrow {
 			 default:
 				dumpage.push_back(JSON_NEW(JSONNode(JSON_TEXT("_type"), JSON_TEXT("Unknown"))));
 		  }
-		  
+
 		  JSONNode str(JSON_NODE);
 		  str.set_name(JSON_TEXT("_name"));
 		  str.push_back(JSON_NEW(JSONNode(json_string(JSON_TEXT("value")), _name)));
 		  str.push_back(JSON_NEW(JSONNode(JSON_TEXT("length"), _name.length())));
 		  str.push_back(JSON_NEW(JSONNode(JSON_TEXT("capactiy"), _name.capacity())));
-		  
+
 		  dumpage.push_back(JSON_NEW(JSONNode(JSON_TEXT("_name_encoded"), _name_encoded)));
 		  dumpage.push_back(JSON_NEW(str));
 		  dumpage.push_back(JSON_NEW(JSONNode(JSON_TEXT("_string_encoded"), _string_encoded)));
@@ -520,7 +788,7 @@ JSONNode ** internalJSONNode::at(const json_string & name_t) json_nothrow {
 		  str.push_back(JSON_NEW(JSONNode(JSON_TEXT("length"), _string.length())));
 		  str.push_back(JSON_NEW(JSONNode(JSON_TEXT("capactiy"), _string.capacity())));
 		  dumpage.push_back(JSON_NEW(str));
-		  
+
 		  if ((type() == JSON_BOOL) || (type() == JSON_NUMBER)){
 			 JSONNode unio(JSON_NODE);
 			 unio.set_name(JSON_TEXT("_value"));
@@ -531,11 +799,11 @@ JSONNode ** internalJSONNode::at(const json_string & name_t) json_nothrow {
 			 }
 			 dumpage.push_back(JSON_NEW(unio));
 		  }
-		  
-		  #if !defined(JSON_PREPARSE) && defined(JSON_READ_PRIORITY) 
+
+		  #if !defined(JSON_PREPARSE) && defined(JSON_READ_PRIORITY)
 			 dumpage.push_back(JSON_NEW(JSONNode(JSON_TEXT("fetched"), fetched)));
 		  #endif
-		  
+
 		  #ifdef JSON_COMMENTS
 			 str.clear();
 			 str.set_name(JSON_TEXT("_comment"));
@@ -544,7 +812,7 @@ JSONNode ** internalJSONNode::at(const json_string & name_t) json_nothrow {
 			 str.push_back(JSON_NEW(JSONNode(JSON_TEXT("capactiy"), _comment.capacity())));
 			 dumpage.push_back(JSON_NEW(str));
 		  #endif
-		  
+
 		  if (isContainer()){
 			 JSONNode arra(JSON_NODE);
 			 arra.set_name(JSON_TEXT("Children"));
@@ -558,7 +826,7 @@ JSONNode ** internalJSONNode::at(const json_string & name_t) json_nothrow {
 			 arra.push_back(JSON_NEW(chil));
 			 dumpage.push_back(JSON_NEW(arra));
 		  }
-		  
+
 		  return dumpage;
 	   }
     #endif
