@@ -5,7 +5,7 @@
 
 /*
     This is the C interface to libjson.
- 
+
     This file also declares various things that are needed for
     C++ programming
 */
@@ -14,7 +14,7 @@
     #ifdef __cplusplus
 	   extern "C" {
     #endif
-		  
+
 		  #ifdef JSON_NO_C_CONSTS
 			 /* The interface has no consts in it, but ther must be const_cast internally */
 			 #define json_const
@@ -23,7 +23,7 @@
 			 #define json_const const
 			 #define TOCONST_CSTR(x) x
 		  #endif
-		  
+
 		  /*
 			 stuff that's in namespace libjson
 		  */
@@ -63,20 +63,21 @@
 		  #ifdef JSON_MEMORY_CALLBACKS
 			 void json_register_memory_callbacks(json_malloc_t mal, json_realloc_t real, json_free_t fre);
 		  #endif
-		  
+
 		  #ifdef JSON_STREAM
-			 JSONSTREAM * json_new_stream(json_stream_callback_t callback);
+			 JSONSTREAM * json_new_stream(json_stream_callback_t callback, json_stream_e_callback_t e_callback, void * identifier);
 			 void json_stream_push(JSONSTREAM * stream, json_const json_char * addendum);
 			 void json_delete_stream(JSONSTREAM * stream);
+		     void json_stream_reset(JSONSTREAM * stream);
 		  #endif
 
-		  
+
 		  /*
 			 stuff that's in class JSONNode
 		   */
 		  /* ctors */
 		  JSONNODE * json_new_a(json_const json_char * name, json_const json_char * value);
-		  JSONNODE * json_new_i(json_const json_char * name, long value);
+		  JSONNODE * json_new_i(json_const json_char * name, json_int_t value);
 		  JSONNODE * json_new_f(json_const json_char * name, json_number value);
 		  JSONNODE * json_new_b(json_const json_char * name, json_bool_t value);
 		  JSONNODE * json_new(char type);
@@ -85,9 +86,9 @@
 
 		  /* assignment */
 		  void json_set_a(JSONNODE * node, json_const json_char * value);
-		  void json_set_i(JSONNODE * node, long value);
+		  void json_set_i(JSONNODE * node, json_int_t value);
 		  void json_set_f(JSONNODE * node, json_number value);
-		  void json_set_b(JSONNODE * node, json_bool_t value); 
+		  void json_set_b(JSONNODE * node, json_bool_t value);
 		  void json_set_n(JSONNODE * node, json_const JSONNODE * orig);
 
 		  /* inspectors */
@@ -99,11 +100,13 @@
 			 json_char * json_get_comment(json_const JSONNODE * node);
 		  #endif
 		  json_char * json_as_string(json_const JSONNODE * node);
-		  long json_as_int(json_const JSONNODE * node);
+		  json_int_t json_as_int(json_const JSONNODE * node);
 		  json_number json_as_float(json_const JSONNODE * node);
 		  json_bool_t json_as_bool(json_const JSONNODE * node);
-		  JSONNODE * json_as_node(json_const JSONNODE * node);
-		  JSONNODE * json_as_array(json_const JSONNODE * node);
+		  #ifdef JSON_CASTABLE
+			 JSONNODE * json_as_node(json_const JSONNODE * node);
+			 JSONNODE * json_as_array(json_const JSONNODE * node);
+		  #endif
 		  #ifdef JSON_BINARY
 			 void * json_as_binary(json_const JSONNODE * node, unsigned long * size);
 		  #endif
@@ -131,7 +134,9 @@
 			 json_char * json_encode64(json_const void * binary, json_index_t bytes);
 			 void * json_decode64(json_const json_char * text, unsigned long * size);
 		  #endif
-		  void json_cast(JSONNODE * node, char type);
+		  #ifdef JSON_CASTABLE
+			 void json_cast(JSONNODE * node, char type);
+		  #endif
 
 		  /* children access */
 		  void json_reserve(JSONNODE * node, json_index_t siz);
@@ -153,7 +158,7 @@
 			 JSONNODE_ITERATOR json_erase_multi(JSONNODE * node, JSONNODE_ITERATOR start, JSONNODE_ITERATOR end);
 			 JSONNODE_ITERATOR json_insert(JSONNODE * node, JSONNODE_ITERATOR it, JSONNODE * node2);
 			 JSONNODE_ITERATOR json_insert_multi(JSONNODE * node, JSONNODE_ITERATOR it, JSONNODE_ITERATOR start, JSONNODE_ITERATOR end);
-	 
+
 			 /* iterator functions */
 			 JSONNODE_ITERATOR json_begin(JSONNODE * node);
 			 JSONNODE_ITERATOR json_end(JSONNODE * node);
@@ -167,44 +172,112 @@
     #endif
 #else
     #ifndef __cplusplus
-	   #error Using the non-library requires C++
+	   #error Turning off JSON_LIBRARY requires C++
     #endif
     #include "Source/JSONNode.h"  //not used in this file, but libjson.h should be the only file required to use it embedded
     #include "Source/JSONWorker.h"
     #include "Source/JSONValidator.h"
     #include "Source/JSONStream.h"
+    #include "Source/JSONPreparse.h"
     #ifdef JSON_EXPOSE_BASE64
-	   #include "JSON_Base64.h"
+	   #include "Source/JSON_Base64.h"
+    #endif
+    #ifndef JSON_NO_EXCEPTIONS
+	   #include <stdexcept>  //some methods throw exceptions
     #endif
-    #include <stdexcept>  //some methods throw exceptions
 
-    namespace libjson {
+	#include <cwchar>  /* need wide characters */
+	#include <string>
+
+    namespace libjson {	   
 	   #ifdef JSON_EXPOSE_BASE64
 		  inline static json_string encode64(const unsigned char * binary, size_t bytes) json_nothrow json_cold {
 			 return JSONBase64::json_encode64(binary, bytes);
 		  }
-		  
+
 		  inline static std::string decode64(const json_string & encoded) json_nothrow json_cold {
 			 return JSONBase64::json_decode64(encoded);
 		  }
 	   #endif
 	   
+	   //useful if you have json that you don't want to parse, just want to strip to cut down on space
+	   inline static json_string strip_white_space(const json_string & json) json_nothrow {
+		  return JSONWorker::RemoveWhiteSpaceAndComments(json, false);
+	   }
+		
+		#ifndef JSON_STRING_HEADER
+			inline static std::string to_std_string(const json_string & str){
+				#if defined(JSON_UNICODE) ||defined(JSON_MEMORY_CALLBACKS) || defined(JSON_MEMORY_POOL)
+					return std::string(str.begin(), str.end());		
+				#else
+					return str;
+				#endif
+			}
+			inline static std::wstring to_std_wstring(const json_string & str){
+				#if (!defined(JSON_UNICODE)) || defined(JSON_MEMORY_CALLBACKS) || defined(JSON_MEMORY_POOL)
+					return std::wstring(str.begin(), str.end());		
+				#else
+					return str;
+				#endif
+			}
+			inline static json_string to_json_string(const std::string & str){
+				#if defined(JSON_UNICODE) ||defined(JSON_MEMORY_CALLBACKS) || defined(JSON_MEMORY_POOL)
+					return json_string(str.begin(), str.end());		
+				#else
+					return str;
+				#endif
+			}
+			inline static json_string to_json_string(const std::wstring & str){
+				#if (!defined(JSON_UNICODE)) || defined(JSON_MEMORY_CALLBACKS) || defined(JSON_MEMORY_POOL)
+					return json_string(str.begin(), str.end());		
+				#else
+					return str;
+				#endif
+			}
+		#endif
+
 	   #ifdef JSON_READ_PRIORITY
 		  //if json is invalid, it throws a std::invalid_argument exception
 		  inline static JSONNode parse(const json_string & json) json_throws(std::invalid_argument) {
-			 return JSONWorker::parse(json);
+			 #ifdef JSON_PREPARSE
+				#if defined JSON_DEBUG || defined JSON_SAFE
+					json_char temp;
+					json_auto<json_char> buffer(JSONWorker::RemoveWhiteSpace(json, temp, false));
+				#else
+					json_auto<json_char> buffer(JSONWorker::RemoveWhiteSpace(json, false));
+				#endif
+				return JSONPreparse::isValidRoot(buffer.ptr);
+			 #else
+				return JSONWorker::parse(json);
+			 #endif
 		  }
-	   
+
 		  inline static JSONNode parse_unformatted(const json_string & json) json_throws(std::invalid_argument) {
-			 return JSONWorker::parse_unformatted(json);
+			 #ifdef JSON_PREPARSE
+				return JSONPreparse::isValidRoot(json);
+			 #else
+				return JSONWorker::parse_unformatted(json);
+			 #endif
 		  }
-		  
+
 		  #ifdef JSON_VALIDATE
 			 inline static bool is_valid(const json_string & json) json_nothrow {
-				return JSONValidator::isValidRoot(JSONWorker::RemoveWhiteSpaceAndComments(json).c_str());
+				#ifdef JSON_SECURITY_MAX_STRING_LENGTH
+				    if (json_unlikely(json.length() > JSON_SECURITY_MAX_STRING_LENGTH)){
+					   JSON_FAIL(JSON_TEXT("Exceeding JSON_SECURITY_MAX_STRING_LENGTH"));
+					   return false;
+				    }
+				#endif
+				return JSONValidator::isValidRoot(JSONWorker::RemoveWhiteSpaceAndComments(json, false).c_str());
 			 }
-			 
+
 			 inline static bool is_valid_unformatted(const json_string & json) json_nothrow {
+				#ifdef JSON_SECURITY_MAX_STRING_LENGTH
+				    if (json_unlikely(json.length() > JSON_SECURITY_MAX_STRING_LENGTH)){
+					   JSON_FAIL(JSON_TEXT("Exceeding JSON_SECURITY_MAX_STRING_LENGTH"));
+					   return false;
+				    }
+				#endif
 				return JSONValidator::isValidRoot(json.c_str());
 			 }
 			 #ifdef JSON_DEPRECATED_FUNCTIONS
@@ -216,19 +289,14 @@
 			 #endif
 		  #endif
 	   #endif
-	   
-	   //useful if you have json that you don't want to parse, just want to strip to cut down on space
-	   inline static json_string strip_white_space(const json_string & json) json_nothrow {
-		  return JSONWorker::RemoveWhiteSpaceAndComments(json);
-	   }	   
-	   
+
 	   //When libjson errors, a callback allows the user to know what went wrong
 	   #if defined JSON_DEBUG && !defined JSON_STDERROR
 		  inline static void register_debug_callback(json_error_callback_t callback) json_nothrow {
 			 JSONDebug::register_callback(callback);
 		  }
 	   #endif
-	   
+
 	   #ifdef JSON_MUTEX_CALLBACKS
 		  #ifdef JSON_MUTEX_MANAGE
 			 inline static void register_mutex_callbacks(json_mutex_callback_t lock, json_mutex_callback_t unlock, json_mutex_callback_t destroy, void * manager_lock) json_nothrow {
@@ -240,12 +308,12 @@
 				JSONNode::register_mutex_callbacks(lock, unlock, manager_lock);
 			 }
 		  #endif
-		  
+
 		  inline static void set_global_mutex(void * mutex) json_nothrow {
 			 JSONNode::set_global_mutex(mutex);
 		  }
 	   #endif
-	   
+
 	   #ifdef JSON_MEMORY_CALLBACKS
 		  inline static void register_memory_callbacks(json_malloc_t mal, json_realloc_t real, json_free_t fre) json_nothrow {
 			 JSONMemory::registerMemoryCallbacks(mal, real, fre);
